require "slidepay"
require "spec_helper"

describe SlidePay::ACH::Retrieval do
  # Test retrieval initialization
  describe "instantiation" do
    it "should populate the object instance with the hash passed in" do
      r = SlidePay::ACH::Retrieval.new(unledgered_retrieval_object)
      expect(r.is_new?).to be_false
      expect(r.id()).to eq(3)
    end
  end

  describe "class methods" do
    describe "create_with_bank_account" do
      # it "should call"
      # 1. [Create] Retrieval with bank_account_id, user_master_id, company_id, location_id
      it "should accept the _id parameters as a hash" do
        params = {
          location_id: 622,
          company_id: 18,
          bank_account_id: 167,
          retrieval_amount: 1.01,
          user_master_id: 661
        }

        SlidePay.should_receive(:post).with(hash_including(data: params.to_json)).and_return(a_response_object)

        SlidePay::ACH::Retrieval.create_with_bank_account(params)
      end

      # 2. [Create] Retrieval with bank_account object
      it "should accept a bank account object as a parameter" do
        bank_account = {
          bank_account_id: 211,
          company_id: 213,
          location_id: 285,
          user_master_id: 297,
          account_first_name: "Father",
          account_last_name: "Yummy Bears",
          name: "ACCOUNT 211",
          routing_number: "211123456",
          account_number: "211211211",
          created: "2013-09-06T04:05:49",
          last_update: "2013-09-06T04:05:49"
        }

        SlidePay.should_receive(:post).with(hash_including(data: bank_account.to_json)).and_return(a_response_object)

        SlidePay::ACH::Retrieval.create_with_bank_account(bank_account)
      end
    end

    describe "retrieve_by_settlement_token" do
      it "should accept a string for the settlement_transaction_token" do
        settlement_transaction_token = "SETTLEMENT_TRANSACTION_TOKEN"
        base_path = "retrieval"
        SlidePay.should_receive(:get).with(hash_including(path: "#{base_path}?settlement_transaction_token=#{settlement_transaction_token}")).and_return(a_response_object)
        SlidePay::ACH::Retrieval.retrieve_by_settlement_token(settlement_transaction_token)
      end

      it "should raise an error if the transaction_token is not a String" do
        expect {
          SlidePay::ACH::Retrieval.retrieve_by_settlement_token(1)
        }.to raise_error
      end
    end
  end

  describe "instance methods" do
    describe "url" do
      it "should append the retrieval_id to url_root if present" do
        r = SlidePay::ACH::Retrieval.new("retrieval_id" => 3)
        expect(r.url()).to eq("retrieval/3")
      end

      it "should append the settlement_transaction_token to url_root if it is present, but retrieval_id is not" do
        r = SlidePay::ACH::Retrieval.new("settlement_transaction_token" => "SETTLEMENT_TRANSACTION_TOKEN")
        expect(r.url()).to eq("retrieval?settlement_transaction_token=SETTLEMENT_TRANSACTION_TOKEN")
      end
    end

    describe "is_new?" do
      it "should return false if settlement_transaction_token is not nil" do
        r = SlidePay::ACH::Retrieval.new("settlement_transaction_token" => "SETTLEMENT_TRANSACTION_TOKEN")
        expect(r.is_new?).to be_false
      end

      it "should return false if retrieval_id is not nil" do
        r = SlidePay::ACH::Retrieval.new("retrieval_id" => 3)
        expect(r.is_new?).to be_false
      end

      it "should return true if neither retrieval_id or settlement_transaction_token are set" do
        r = SlidePay::ACH::Retrieval.new
        expect(r.is_new?).to be_true
      end
    end

    describe "create_with_bank_account" do
      it "should call the class-level create_with_bank_account method" do
        bank_account_hash = {}
        SlidePay::ACH::Retrieval.should_receive(:create_with_bank_account).and_return({})

        r = SlidePay::ACH::Retrieval.new()
        r.create_with_bank_account(bank_account_hash)
      end

      it "should pass auth options into the class-level method" do
        auth_hash = { token: "TOKEN", api_key: "API_KEY", endpoint: "ENDPOINT" }
        SlidePay::ACH::Retrieval.should_receive(:create_with_bank_account).with({}, auth_hash).and_return({})

        r = SlidePay::ACH::Retrieval.new(auth_hash)
        r.create_with_bank_account({})
      end
    end

    describe "retrieve_by_settlement_token" do
      it "should call the class-level retrieve_by_settlement_token method" do
        transaction_token = "SETTLEMENT_TRANSACTION_TOKEN"
        SlidePay::ACH::Retrieval.should_receive(:retrieve_by_settlement_token) do |arg1, arg2|
          expect(arg1).to eq(transaction_token)
          {}
        end

        r = SlidePay::ACH::Retrieval.new()
        r.retrieve_by_settlement_token(transaction_token)
      end

      it "should pass auth options into the class-level method" do
        auth_hash = { :token => "TOKEN", :api_key => "API_KEY", :endpoint => "ENDPOINT" }
        transaction_token = "SETTLEMENT_TRANSACTION_TOKEN"

        SlidePay::ACH::Retrieval.should_receive(:retrieve_by_settlement_token).with(transaction_token, auth_hash).and_return({})

        r = SlidePay::ACH::Retrieval.new(auth_hash)
        r.retrieve_by_settlement_token(transaction_token)
      end
    end

    # Test retrieve
    describe "retrieve" do
      before(:all) do
        SlidePay.stub(:get) { a_response_object(successful_retrieval_retrieve_response) }
      end

      it "should populate a retrieval object on success" do
        r = SlidePay::ACH::Retrieval.new()
        r["retrieval_id"] = 4
        r.retrieve()

        expect(r.is_new?).to be_false
        expect(r["settlement_transaction_token"]).to eq("B41BB60D-942C-46DB-99F8-D3308F3ECA3D")
      end
    end

    describe "ledgered?" do
      it "should return false if the retrieval has not been ledgered" do
        r = SlidePay::ACH::Retrieval.new(unledgered_retrieval_object)
        expect(r.is_ledgered?).to be_false
      end

      it "should return true if the retrieval has been ledgered" do
        r = SlidePay::ACH::Retrieval.new(ledgered_retrieval_object)
        expect(r.is_ledgered?).to be_true
      end
    end
  end
end

